//
// JUMA-TRX2 protocol
//
// Juha Niinikoski, OH2NLT 06.07.2006
//
//


#include "juma-trx2.h"				// board definitions

// module variables

#define CMD_BUF_SIZE 12
static char cmd_buf[CMD_BUF_SIZE];	// command buffer
static int cmd_buf_idx;				// buffer fill pointer



// external references
// external functions

extern void putch(char);						// uart I/O
extern int kbhit(void);
extern unsigned char getch(void);
extern unsigned char getche(void);
extern unsigned long getlong(void);
extern void bin2ascii(unsigned long);
extern void putlong( unsigned long); 

extern int PTT_IN;							// atomic copy of PTT_IN_IO input / TX on, 0 = active
extern int s_meter;							// scaled S-meter value 0 ... 48
extern int pwr_meter;						// scaled PWR-meter value 0 ... 48
extern int vfo_idx;							// VFO index, A=0, B=1, if(Split) then  RX=0, TX=1
extern long freq_display;					// display frequency, no RIT added

// SCAF filter
extern const int scaf_min[3];				// filter tuning limits
extern const int scaf_max[3];

// stuff from Yaesu Cat module
#define DDS_MULT 102481911521LL 			// 2^32/clkin * 2^32, for 180000000Hz nominal clock
extern long long dds_mult;					// calculate with ref osc calibration value

// EEPROM structures

#include "trx2_eeprom.h"				// get EEPROM structure definitions

extern union
	{
	struct defval defval;
	unsigned int storage[sizeof(struct defval)/2];
	}eeprom;

// system calibration values

extern union
	{
	struct calval calval;
	unsigned int ee[sizeof(struct calval)/2];
	}cal;


extern int rf_filt_idx;					// TRX2 band data
extern long freq_in_use;				// frequency (DDS set value) currently in use

extern unsigned int pa100_tx_tmr;		// PA100 max band change rate

#define MAX_TX_RATE	100					// max TX rate (ms)

int old_rf_filt_idx;					// used to detect the changes

// Some I/O functions
// Special output functions

static unsigned char disp_buffer[9];		// display buffer for long number printouts

/* Convert binary number to ASCII string */
/* 8 digits are converted */

void bin2asc(unsigned long luku)
	{
char x, idx;

	idx = 7;
	for(x=0; x<8; x++)				// do 8 digits
		{
		disp_buffer[idx] = (luku % 10) + '0';
		luku = luku / 10;
		idx--;
		}
	disp_buffer[8] = 0;			// string terminator
	}

void put_freq( unsigned long x)
	{
	bin2asc(x);					// partial convert long number to string

	putch(disp_buffer[0]);		// MSB
	putch(disp_buffer[1]);		// 

	putch(disp_buffer[2]);		// 
	putch(disp_buffer[3]);		//
	putch(disp_buffer[4]);		//

	putch(disp_buffer[5]);		//
	putch(disp_buffer[6]);		//
	putch(disp_buffer[7]);		// LSB
	}

// convert number from command buffer
unsigned long get_number(void)
	{
	unsigned long n;
	int i;
	i = 2;								// msb of the number
	n= 0;

	while(i < cmd_buf_idx)				// cmd_buf_idx point to last(lsb) digit +1
		{
		n = (n * 10) + (cmd_buf[i] & 0x0F);
		i++;
		}
	return(n);
	}


// TRX2 protocol parser

// Send band data
void send_band_data(void)
	{
	int i;
	i = rf_filt_idx - 2;				// addjust TRX2 filter index to match PA100 band select
	if(i < 0)
		i = 0;
	printf("=B%i\n\r", i);
	}

// send S-meter value
void send_s_meter(void)
	{
	if(PTT_IN == 0)										// TXon ?
		printf("=S%i\n\r", pwr_meter); 					// PWR meter
	else
		printf("=S%i\n\r", s_meter); 					// S-meter
	}

// Send mode data
void send_mode(void)
	{
	printf("=M%i\n\r", eeprom.defval.mode[vfo_idx]);					// send mode data
	}

// Send VFO number in use
void send_VFO(void)
	{
	printf("=V%i\n\r", eeprom.defval.vfo_sel);							// send VFO #
	}

// Send frequency
void send_FREQ(void)
	{
	unsigned long l;
	long long v;					// very long variable for frequency display calculations

	v = (long long)freq_display * (long long)cal.calval.ref_osc;	  	// calculate display frequency 
	l = v >> 33;														// display LO frequency = DDS freq / 2

	printf("=F");
	put_freq(l);														// print frequency
	printf("\n\r");
	}		

// Send selected audio filter
void send_filter(void)
	{
	printf("=A%i\n\r", eeprom.defval.filter);							// send audio filter #
	}

// Send selected filter bandwitdh
void send_filter_width(void)
	{
	printf("=W%li\n\r",(75000L/eeprom.defval.scaf_filters[eeprom.defval.filter]));	// convert to Hz
	}

// send RIT status
void send_rit()
	{
		printf("=R%i\n\r",eeprom.defval.rit_on);						// RIT status, 0 = off, 1 = on
	}


// Set frequency
void set_FREQ(void)
	{
	unsigned long ultemp;
	long long vltemp;

	ultemp = get_number();								// get decimal frequency
	ultemp = ultemp + 5;								// round
	if(ultemp > 40000000)								// check limits
		ultemp = 40000000;

// calculate DDS control word
// DDS control word N = ((F in * dds_mult) >> 32) * 2
// or N = (F in * dds_mult) >> 31

	vltemp = (long long)ultemp * dds_mult;				// dds_mult calculated in main
	eeprom.defval.vfo[vfo_idx] = vltemp >> 31;			// set frequency to selected VFO
	}

// Set audio filter number
void set_filter(void)
	{
	int b;
	b = cmd_buf[2] & 0x0F;							// simple ASCII to number conversion
	if(b > 2)										// stay in range
			b = 2;
	eeprom.defval.filter = b;						// Select filter
	}

// set selected filter width
void set_filter_width(void)
	{
	int w;

	w = get_number();																					// get filter width (Hz)
	eeprom.defval.scaf_filters[eeprom.defval.filter] = 75000L / w;										// convert to timer units

	if(eeprom.defval.scaf_filters[eeprom.defval.filter] < scaf_min[eeprom.defval.filter])				// check limits
		eeprom.defval.scaf_filters[eeprom.defval.filter] = scaf_min[eeprom.defval.filter];
	if(eeprom.defval.scaf_filters[eeprom.defval.filter] > scaf_max[eeprom.defval.filter])
		eeprom.defval.scaf_filters[eeprom.defval.filter] = scaf_max[eeprom.defval.filter];
	}

// set RIT status
void set_rit()
	{
	int b;
	b = cmd_buf[2] & 0x0F;							// simple ASCII to number conversion
	if(b > 1)										// stay in range
			b = 1;
	eeprom.defval.rit_on = b;						// RIT On / Off
	}

// clear buffer
void clear_buffer(void)
	{
	int x;
	for(x=0; x<CMD_BUF_SIZE; x++)
		{
		cmd_buf[x] = 0;
		}
	cmd_buf_idx = 0;
	}

// filter no operation characters, return = 1 if no op
int filter_nop_char(char c)
	{
	if((c==0x00) || (c==0x0A) || (c==0x20) || (c==0x2C) || (c==0x2E))
		return(1);
	else
		return(0);
	}

// Parse TRX2 message
void serial_trx2(void)
	{
	char c;
	int b;

// detect TRX2 band changes & send band data to the PA100
	if(rf_filt_idx != old_rf_filt_idx)				// mand changed
		{
		if(pa100_tx_tmr == 0)
			{
			pa100_tx_tmr = MAX_TX_RATE;				// load inhibit timer
			old_rf_filt_idx = rf_filt_idx;			// save current situation
			send_band_data();						// send band data
			}
		}

// process input
	if(kbhit())
		{
		c = getch();								// save character

// filter nop
		if(filter_nop_char(c))						// discard nop characters 
			return;									// do nothing

// capture start sync
	if(c == '=' || c == '?')						// sync with action characters
		clear_buffer();

// check if buffer ready, interprete message
		if(c == 0x0D)								// CR = ready
			{
			switch(cmd_buf[0])						// action
				{
				case '?':							// query
					switch(cmd_buf[1])
						{
						case 'B':
							send_band_data();		// send band data answer
						break;

						case 'S':
							send_s_meter();			// send S-meter value
						break;

						case 'M':
							send_mode();			// send mode data
						break;
		
						case 'V':
							send_VFO();				// send VFO # in use
						break;

						case 'F':
							send_FREQ();			// send frequency in use
						break;

						case 'A':
							send_filter();			// send selected filter
						break;

						case 'W':
							send_filter_width();	// send selected filter bandwith
						break;

						case 'R':
							send_rit();				// send RIT status
						break;
						}

					clear_buffer();					// clear for next round
				break;

				case '=':							// set
					switch(cmd_buf[1])
						{
						case 'M':
							b = cmd_buf[2] & 0x0F;					// simple ASCII to number conversion
							if(b > 3)								// stay in range
								b = 3;
							eeprom.defval.mode[vfo_idx] = b;		// Set mode data
						break;
		
						case 'V':
							b = get_number();						// get VFO number
							if(b > 25)								// > Z
								b = 25;
							eeprom.defval.vfo_sel = b;				// Set VFO # in use
						break;

						case 'F':
							set_FREQ();								// set frequency in selected VFO
						break;

						case 'A':
							set_filter();							// send selected filter
						break;

						case 'W':
							set_filter_width();						// set selected filter bandwith
						break;

						case 'R':
							set_rit();								// set RIT on/off
						break;
						}

					clear_buffer();									// clear for next round
				break;

				default:											// not known
					clear_buffer();									// clear for next round
				break;
				}
			}
		else
			{
// collect data
		cmd_buf[cmd_buf_idx] = c;		// save data
		if(cmd_buf_idx < CMD_BUF_SIZE-1)
			cmd_buf_idx++;				// addjust index
			}
		}
	}
